require "defines"
require "config"
require "mod_util"

-- On game creation, setup appropriate evolution scaling
game.on_init( function()
	if ENABLE_EVOLUTION_SCALING then
		RebuildEvolutionFactors()
	end
end)

-- On game load, if enabled, rebuild tech and/or evolution
game.on_load( function()
	if TECH_REBUILD_ON_LOAD then
		game.player.force.resettechnologies()
		
		CheckTechErrors()-- If enabled and appropriate, display mod feedback
	end
	
	if EVOLUTION_REBUILD_ON_LOAD then
		RebuildEvolutionFactors()
		
		CheckEvolutionErrors()
	end
end)

-- When the player starts a new game, if enabled and appropriate, display mod feedback
game.on_event(defines.events.on_player_created, function()
	CheckTechErrors()
	CheckEvolutionErrors()
end)

-- Making appropriate changes to the evolution values
function RebuildEvolutionFactors()
	local evolution = game.mapsettings.enemy_evolution -- Shallow copy for readability
	
	-- Ensuring active evolution scalings are valid
	if MANUAL_EVOLUTION_OVERRIDE and EvolutionInputsValid() then
		evolution.time_factor = evolution.time_factor * MANUAL_EVOLUTION_SCALE_TIME
		evolution.destroy_factor = evolution.destroy_factor * MANUAL_EVOLUTION_SCALE_DESTROY
		evolution.pollution_factor = evolution.pollution_factor * MANUAL_EVOLUTION_SCALE_POLLUTION
	elseif TechInputsValid() then
		-- Automatically scale evolution down as tech count and time scale up
		local tempScale = 1
		
		if ENABLE_TECH_COUNT_SCALING then
			tempScale = tempScale / TECH_COUNT_SCALE
		end
		
		if ENABLE_TECH_TIME_SCALING then
			tempScale = tempScale / TECH_TIME_SCALE
		end
		
		evolution.time_factor = evolution.time_factor * tempScale
		evolution.destroy_factor = evolution.destroy_factor * tempScale
		evolution.pollution_factor = evolution.pollution_factor * tempScale
	end
end

-- Error messages for data rebuilding
function CheckTechErrors()
	if ENABLE_FEEDBACK_MESSAGES and
	  (ENABLE_TECH_COUNT_SCALING or ENABLE_TECH_TIME_SCALING or ENABLE_TECH_RECIPE_SCALING) then
		if TechInputsValid() then
			game.player.print("Scalar Mod: Technologies have been rebuilt.")
		else
			game.player.print("Scalar Mod ERROR: Technologies have not been rebuilt. One or more inputs were invalid. Previous values retained.")
		end
	end
end

function CheckEvolutionErrors()
	if ENABLE_FEEDBACK_MESSAGES and ENABLE_EVOLUTION_SCALING then
		if (not MANUAL_EVOLUTION_OVERRIDE and TechInputsValid()) or
		   (MANUAL_EVOLUTION_OVERRIDE and EvolutionInputsValid()) then
			game.player.print("Scalar Mod: Alien evolution values have been rebuilt.")
		else
			game.player.print("Scalar Mod ERROR: Alien evolution values have not been rebuilt. One or more inputs were invalid. Previous values retained.")
		end
	end
end